// ExtraExtra!  Simulate a spinning newspaper effect
// from classic movies.  This sketch uses fairly advanced
// ideas, but you're not required to understand them at
// this point in the course.  Just make sure you find the
// things you are responsible for changing.

// The template image of a newspaper page.
PImage page_img;
// The image to paste in for the main story.
PImage portrait_img;

// An offscreen buffer into which we'll render
// the assembled newspaper image (template,
// portrait, and headlines)
PGraphics assembled;

// A bunch of carefully computed measurements that
// depend on the precise layout of the template
// image.
int pbox_width = 192;
int pbox_height = 203;
int pbox_x = 202;
int pbox_y = 212;
int caption_x = pbox_x;
int caption_y = 428;
int headline_x = 28;
int headline_y1 = 150;
int headline_y2 = 195;

float pbox_ratio = (float)pbox_width / (float)pbox_height;

// TODO -- put your name in the string below.
String name = "<YOUR NAME HERE>";
// TODO -- put your picture in the file portrait.jpg,
// or change the filename below to your filename.
String portrait_fname = "portrait.jpg";

// How many frames to spin the image for before stopping.
int num_frames = 300;

void setup()
{
  size( 800, 600, P2D );
  background( 0 );

  page_img = loadImage( "newspaper.png" );

  // Try to load in the portrait image.  Only compose it onto
  // the newspaper if it's found.
  portrait_img = loadImage( portrait_fname );
  if ( portrait_img != null ) {
    portrait_img.filter( GRAY );
    float aspect = (float)portrait_img.width / (float)portrait_img.height;

    // Resize the image to fit the box on the newspaper page.
    // Choose how best to resize it based on its aspect ratio.
    if ( aspect > pbox_ratio ) {
      portrait_img.resize( 0, pbox_height );
      int w = portrait_img.width;
      portrait_img = portrait_img.get(
        w/2 - pbox_width/2, 0, pbox_width, pbox_height );
    } else {
      portrait_img.resize( pbox_width, 0 );
      int h = portrait_img.height;
      portrait_img = portrait_img.get(
        0, h/2 - pbox_height/2, pbox_width, pbox_height );
    }
  }

  assembled = createGraphics( page_img.width, page_img.height );
  assembled.beginDraw();
  assembled.image( page_img, 0, 0 );
  if ( portrait_img != null ) {
    assembled.image( portrait_img, pbox_x, pbox_y );
  }

  // Add the headline and the image caption.
  assembled.fill( 0 );
  assembled.textFont( createFont( "TimesNewRomanPS-BoldMT", 10 ) );
  assembled.text( name + " in Waterloo, Ontario", caption_x, caption_y );
  assembled.textFont( createFont( "TimesNewRomanPS-BoldMT", 50 ) );
  assembled.text( name.toUpperCase(), headline_x, headline_y1 );
  assembled.text( "TO ENROLL IN CS 106", headline_x, headline_y2 );

  assembled.noFill();

  // Add a gradient to the bottom of the page to suggest a
  // curve to the folded newspaper.
  for ( int idx = 0; idx < 30; ++idx ) {
    assembled.stroke( 0, 0, 0, map( idx, 0, 29, 250, 0 ) );
    assembled.line( 0, assembled.height - 1 - idx, assembled.width, 
      assembled.height - 1 - idx );
  }

  assembled.endDraw();

  fill( 0, 30 );
}

void draw()
{
  if ( frameCount >= num_frames ) {
    // If we're done spinning, just draw the final newspaper
    // in place.

    rect( 0, 0, width, height );
    pushMatrix();
    translate( width/2, height/2 );
    translate( -assembled.width/2, -assembled.height/2 );

    image( assembled, 0, 0 );
    popMatrix();

    if ( frameCount >= num_frames + 50 ) {
      noLoop();
    }
  } else {
    // Otherwise, draw the spinning effect.  This is achieved by
    // drawing multiple superimposed, partially transparent copies
    // of the image to create the appearance of motion blur.
    
    for ( int idx = 0; idx < 4; ++idx ) {
      rect( 0, 0, width, height );

      pushMatrix();
      translate( width/2, height/2 );
      rotate( map( frameCount + idx/4.0, 0, num_frames, 0, 28.0*PI ) );
      scale( pow( map( frameCount + idx/4.0, 0, num_frames, 0, 1 ), 3 ) );
      translate( -assembled.width/2, -assembled.height/2 );

      image( assembled, 0, 0 );
      popMatrix();
    }
  }
}